@@ -1,14 +1,15 @@
 #!/usr/bin/env python
 
+import csv
+import json
 from os.path import isfile
 from os.path import basename
 from os.path import join  as pjoin
-import csv
 from pprint import pprint as pp
 from typing import Dict, Tuple, List, Union, Any, Optional, IO
 
 import datagristle.common as comm
-from datagristle.common import abort as abort
+from datagristle.common import abort
 import datagristle.csvhelper as csvhelper
 
 OUTPUT_TYPES = ['insert', 'delete', 'same', 'chgnew', 'chgold']
@@ -18,7 +19,7 @@ RecordType = List[str]
 
 
 
-class FileDelta(object):
+class FileDelta:
     """" Compares two files based on a common key writing results to multiple
          output files.
 
@@ -114,7 +115,9 @@ class FileDelta(object):
 
         """
         self.dry_run = dry_run
-        # validate inputs
+        self.new_fqfn = new_fqfn
+        self.old_fqfn = old_fqfn
+
         assert isfile(old_fqfn)
         assert isfile(new_fqfn)
 
@@ -139,29 +142,30 @@ class FileDelta(object):
         self._read_old_csv()
         self._read_new_csv()
 
-        while self.old_rec or self.new_rec:
-            if not self.old_rec: # old file is eof
-                self._writer('insert', self.new_rec)
-                self._read_new_csv()
-            elif not self.new_rec: # new file is eof
+        while self.old_rec and self.new_rec:
+            key_match_result = self._key_match()
+            if key_match_result == 'new-greater':
                 self._writer('delete', self.old_rec)
                 self._read_old_csv()
+            elif key_match_result == 'old-greater':
+                self._writer('insert', self.new_rec)
+                self._read_new_csv()
             else:
-                key_match_result = self._key_match()
-                if key_match_result == 'new-greater':
-                    self._writer('delete', self.old_rec)
-                    self._read_old_csv()
-                elif key_match_result == 'old-greater':
-                    self._writer('insert', self.new_rec)
-                    self._read_new_csv()
+                if self._data_match(ignore_fields, compare_fields):
+                    self._writer('same', self.old_rec)
                 else:
-                    if self._data_match(ignore_fields, compare_fields):
-                        self._writer('same', self.old_rec)
-                    else:
-                        self._writer('chgold', self.old_rec)
-                        self._writer('chgnew', self.new_rec)
-                    self._read_new_csv()
-                    self._read_old_csv()
+                    self._writer('chgold', self.old_rec)
+                    self._writer('chgnew', self.new_rec)
+                self._read_new_csv()
+                self._read_old_csv()
+
+        while self.new_rec:
+            self._writer('insert', self.new_rec)
+            self._read_new_csv()
+
+        while self.old_rec:
+            self._writer('delete', self.old_rec)
+            self._read_old_csv()
 
         old_f.close()
         new_f.close()
@@ -176,18 +180,18 @@ class FileDelta(object):
             in_fn: input file name
             out_type: one of insert, delete, same, chgnew, chgold
         Returns:
-            out_fn: which is the in_fn, with .uniq or .sorted.uniq removed,
+            out_fn: which is the basename of in_fn, with .uniq or .sorted.uniq removed,
                     then with out_type added
         Notes:
             parsing is fragile should improve
         """
         fn = basename(in_fn)
         if fn.endswith('.sorted.uniq'):
-            out_fn = in_fn[:-12]  #todo: make less fragile
+            out_fn = fn[:-12]  #todo: make less fragile
         elif fn.endswith('.uniq'):
-            out_fn = in_fn[:-5]   #todo: make less fragile
+            out_fn = fn[:-5]   #todo: make less fragile
         elif fn.endswith('.sorted'):
-            out_fn = in_fn[:-7]   #todo: make less fragile
+            out_fn = fn[:-7]   #todo: make less fragile
         else:
             out_fn = fn
         return out_fn + '.' + out_type
@@ -254,14 +258,14 @@ class FileDelta(object):
             if last_rec is None: # first read priming
                 last_rec = self.new_rec
             if len(last_rec) != len(self.new_rec):
-                abort('new file has inconsistent number of fields')
+                abort('new file has inconsistent number of fields', f'new_rec = {self.new_rec}')
             for key in self.join_fields:
                 if self.new_rec[key] > last_rec[key]:
                     self.new_read_cnt += 1
                     break # good
                 if self.new_rec[key] < last_rec[key]:
-                    print(self.new_rec)
-                    abort('new file is not sorted correctly')
+                    abort('ERROR: new file is not sorted correctly',
+                          f'This refers to file {self.new_fqfn}, and key: {key}, and record: {self.new_rec} and last rec: {last_rec}')
         except StopIteration:
             self.new_rec = None
 
@@ -280,14 +284,14 @@ class FileDelta(object):
             if last_rec is None: # first read priming
                 last_rec = self.old_rec
             if len(last_rec) != len(self.old_rec):
-                abort('old file has inconsistent number of fields')
+                abort('old file has inconsistent number of fields', f'old_rec = {self.new_rec}')
             for key in self.join_fields:
                 if self.old_rec[key] > last_rec[key]:
                     self.old_read_cnt += 1
                     break # good
                 if self.old_rec[key] < last_rec[key]:
-                    print(self.old_rec)
-                    abort('old file is not sorted correctly')
+                    abort('ERROR: old file is not sorted correctly',
+                          f'This refers to file {self.old_fqfn}, and key: {key}, and record: {self.old_rec} and last rec: {last_rec}')
         except StopIteration:
             self.old_rec = None
 
@@ -310,7 +314,7 @@ class FileDelta(object):
 
 
 
-class DeltaAssignments(object):
+class DeltaAssignments:
     """ Manages the post-delta transformations (aka assignments).
     """
 
@@ -345,7 +349,7 @@ class DeltaAssignments(object):
             assert int(dest_field)
         if src_field:
             assert int(src_field)
-        if dest_file not in ['insert', 'delete', 'chgold', 'chgnew']:
+        if dest_file not in ['insert', 'delete', 'chgold', 'chgnew', 'same']:
             raise ValueError('Invalid dest_file: %s' % dest_file)
         if not comm.isnumeric(dest_field):
             raise ValueError('Invalid dest_field: %s' % dest_field)
@@ -434,8 +438,9 @@ class DeltaAssignments(object):
                         outrec[dest_field] = self._get_seq_value(assigner['src_field'])
                     elif assigner['src_type'] == 'special':
                         outrec[dest_field] = self._get_special_value(assigner['src_val'])
-                except ValueError as err:
-                    abort(err)
+                except (ValueError, IndexError) as err:
+                    msg = f'assignments={assigner}, dest_field={dest_field}, outtype={outtype}'
+                    abort(repr(err), msg, verbosity='debug')
         return outrec
 
 
@@ -451,8 +456,7 @@ class DeltaAssignments(object):
         try:
             return self.special_values[src_val]
         except KeyError:
-            pp(self.special_values)
-            abort('Invalid special value referenced in assignment: %s' % src_val)
+            abort(f'Invalid special value in assignment: {src_val}', json.dumps(self.special_values))
 
 
     def _get_copy_value(self, src_file: str, src_field: int) -> str:
@@ -465,10 +469,10 @@ class DeltaAssignments(object):
         Raises:
             ValueError if args are invalid
         """
-        if not self.old_rec:
-            raise ValueError('Assign-Copy refers to non-existing old_rec - invalid config')
         try:
             if src_file == 'old':
+                if not self.old_rec:
+                    raise ValueError('Assign-Copy refers to non-existing old_rec - invalid config')
                 return self.old_rec[src_field]
             elif src_file == 'new':
                 return self.new_rec[src_field]
